﻿#
# Powershell Script zum exportieren der Virtuellen Maschinen. Es wird die Powershell "PowerShell management Library for Hyper-V"
# benutzt (zu finden http://pshyperv.codeplex.com/)
# Version 1.00 vom 12.7.2011 by c.rachfahl@rachfahl.de 
#
# Diese Script wird bereitgestellt wie es ist, ohne jegliche Garantie. Der Einsatz
# erfolgt auf eigene Gefahr. Es wird jegliche Haftung ausgeschlossen.
#
# THIS CODE IS MADE AVAILABLE AS IS, WITHOUT WARRANTY OF ANY KIND. THE ENTIRE RISK
# OF THE USE OR THE RESULTS FROM THE USE OF THIS CODE REMAINS WITH THE USER.
#
# 
#

Param(
	[string] $VM ="",
	[string] $Server =".",
	[string] $ExportPath = "",
	[string] $RemotePath = "",
	[int]    $timeout = 600,
	[switch] $SaveState,
	[switch] $TurnOff,
	[int]    $StartDelay = 0,
	[switch] $verbose
)

# This function validates the scripts parameters
function ValidateParams
{
  $validInputs = $true
  $sleep = 0;
  $errorString =  ""

  if ($VM -eq "")
  {
    $validInputs = $false
    $errorString += "`nFehlender Parameter: Der -VM Parameter ist notwendig. Bitte geben Sie den Namen der VM an die exportiert werden soll."
  }

  if ($Server -ne ".")
  {
	try {
		$VMs = Get-VM $Server
	}
	catch {
    	$validInputs = $false
    	$errorString += "`nFalscher Parameter: Kann auf Server $Server nicht zugreifen."
	}
  }

  if ($ExportPath -eq "")
  {
    $validInputs = $false
    $errorString += "`nFehlender Parameter: Der -ExportPath Parameter ist notwendig. Bitte geben Sie den Pfad an dem der Export lokal gespeichert werden soll."
  } else {
	if ($Server -ne ".") {
  		# da es ein RemoteSystem ist versuchen wir über die AdminFreigabe zu schauen ob ExportPath existiert
		$UNCPath = "\\" + $Server + "\" + $ExportPath.Replace(":", "$")
		$pathOK = Test-Path $UNCPath
	} Else {
 		$pathOK = Test-Path $ExportPath	
	}
	if ($pathOK -ne $true) {
		$validInputs = $false
    	$errorString += "`nFalscher Parameter: Auf den ExportPath $ExportPath kann nicht zugegriffen werden."	
	}
  }

  if ($RemotePath -ne "")
  {
    # ist RemotePath ein zugreifbarer UNC Pfad
  	$pathOK = Test-Path $RemotePath
	if ($pathOK -ne $true) {
		$validInputs = $false
    	$errorString += "`nFalscher Parameter: Auf den RemotePath $RemotePath kann nicht zugegriffen werden."	
	}
  }


  if ($StartDelay -notmatch "^\d+$") {
	$validInputs = $false
        $errorString += "`nFalscher Parameter: -StartDeleay Parameter ist keine Zahl."	
  }


  if (!$validInputs)
  {
    Write-Warning "$errorString"
  }

  return $validInputs
}

# Function that returns true if the incoming argument is a help request
function IsHelpRequest
{
	param($argument)
	return ($argument -eq "-?" -or $argument -eq "-help");
}

# Function that displays the help related to this script following
# the same format provided by get-help or <cmdletcall> -?
function Usage
{
@"

NAME:
`tHyperV-Backup.ps1 (Version 1.0)

SYNOPSIS:
`tScript fährt eine VM herunter, exportiert diese und startet sie dann
`twieder.

SYNTAX:
`tHyperV-Backup.ps1
`t`t[-VM <VM Name>]
`t`t[-Server <ServerIdParameter>]
`t`t[-ExportPath <Verzeichnis>]
`t`t[-RemotePath <UNCPfad>]
`t`t[-SaveState]
`t`t[-NoStart]
`t`t[-startupdelay <Sekunden>]
`t`t[-verbose]


PARAMETER:
`t-VM (benötigt)
`t`tName der VM die exportiert werden soll.

`t-Server (optional)
`t`tDer Server auf dem die VM liegt. Wenn Parameter nicht angegeben
`t`twird, dann wird der lokale Server benutzt.

`t-ExportPath (benötigt)
`t`tLokales Verzeichnis in den die VM exportiert wird.

`t-RemotePath (optional)
`t`tNetzwerk Pfad auf den VM kopiert wird.

`t-SaveState (optional)
`t`tSystem wird nicht heruntergefahren sondern nur gespeichert.
`t`tNützlich bei Systemen die keine Hyper-V Treiber installiert haben.

`t-TurnOff (optional)
`t`tSystem wird nach Export ausgeschaltet.

`t-StartDeleay <Sekunden> (optional)
`t`tSystemstart wird um <Sekunden> Sekunden verzögert.

`t-verbose (optional).
`t`tScript gibt Debuging Informationen aus.


`t-------------------------- Beispiel 1 --------------------------

C:\PS> .\HyperV-Backup.ps1 -VM "W2K8R2" -ExportPath "C:\Exports" 

`t-------------------------- Beispiel 2 --------------------------

C:\PS> .\HyperV-Backup.ps1 -VM "W2K8R2" -Server Hyperv5 -ExportPath "C:\Exports" -RemotePath "\\Storage2\HypervBackups" -verbose

"@
}

# Fuktion lädt PSHyperV Library (aber nur wenn sie nicht schon geladen ist)
#
function LoadPSHyperV 
{
	try {
		$VMs = Get-VM
	}
	catch {
		if($verbose -eq $true) { Write-Output "Lade HyperV Library" }
			Import-Module HyperV
	}
	return
}
####################################################################################################
# Script starts here
####################################################################################################

# Check for Usage Statement Request
$args | foreach { if (IsHelpRequest $_) { Usage; exit; } }

#ist PShyperV Libary schon importiert?
$PSHyperVLoaded = LoadPSHyperV;

# Validate the parameters
$ParamsOK = ValidateParams;
if (!$ParamsOK) { exit; }

# Zeit ausgeben
if ($verbose -eq $true) { Write-Output (Get-Date) "Export der VM $VM gestartet" }

# hohle VM
$myVm = Get-VM -Server $Server -Name $VM
if ($myVm.VMElementName.length -lt 0) {
	if($verbose -eq $true) { Write-Output "VM $VM gibt es nicht" }
	exit
}

# Hat die VM die Installationskomponeten installiert?
$vmHeartBeat = Test-VMHeartBeat $myVm
if($vmHeartBeat.Status -ne "OK") {
	if($verbose -eq $true) { Write-Output "Keine Hyper-V Komponenten gefunden" }
	$SaveState = $true
}

# ist VM im Status Running?
if ($myVm.EnabledState -eq $VMState.Running) {
	if ($verbose -eq $true) { Write-Output "VM $VM ist im Status Running" }
	# soll sie nur schafen?
	if ($SaveState -eq $true) {
		#lege Sie schlafen
		Save-VM $myVm -Server $Server -Force -Wait
	} else {
		#VM herunterfahren
		Invoke-VMShutdown $myVm -Server $Server -Force -ShutdownTimeOut $timeout
	}
} elseif ($myVm.EnabledState -eq $VMState.Suspended -or $myVm.EnabledState -eq $VMState.Stopped) {
	if ($verbose -eq $true) { Write-Output "VM $VM ist im Status Suspended oder Stopped"}
	# soll sie nur schafen?
	if ($SaveState -eq $true -and $myVm.EnabledState -eq $VMState.Suspended) {
		if ($verbose -eq $true) { Write-Output "VM $VM ist im Status Suspended"}	
	} else {
		#VM herunterfahren
		Invoke-VMShutdown $myVm -Server $Server -Force -ShutdownTimeOut $timeout
	} 
}


# Ist VM exportierbar?
$myVm = Get-VM -Server $Server -Name $VM
#$StatusI = $myVm.EnabledState
#if ($verbose -eq $true) { Write-Output "VM $VM ist im Status $StatusI "}

#Wenn VM Remote ist dann erzeige UNC_Pfad auf das Verzeichnis
if($Server -ne "." ) {
	$UNCPath = "\\" + $Server + "\" + $ExportPath.Replace(":", "$")
}

if ($myVm.EnabledState -eq $VMState.Suspended -or $myVm.EnabledState -eq $VMState.Stopped) {
	#lösche vorhandes Verzeichnis
	if($Server -eq "." ) {
		if(Test-Path $ExportPath\$VM) {
			if($verbose -eq $true) { Write-Output "Lösche Verzeichnis $ExportPath\$VM" }
			Remove-Item $ExportPath\$VM -recurse
		}
	} elseif ($Server -ne ".") {
		if(Test-Path $UNCPath\$VM) {
			if($verbose -eq $true) { Write-Output "Lösche Verzeichnis $UNCPath\$VM" }
			Remove-Item $UNCPath\$VM -recurse
		}
	}
	#exportiere VM
	if($verbose -eq $true) { Write-Output "Exportiere $VM in Verzeichnis $ExportPath\$VM" }	
 	Export-VM $myVm -path $ExportPath -copyState -Wait
} else {
	Write-Warning "$VM wurde nicht heruntergefahren und kann deshalb nicht exportiert werden"
}

# starte VM wieder
if ($TurnOff -ne $true) {
	if ($myVm.EnabledState -ne $VMState.Running) {
		# verzögere Start 
		if($verbose -eq $true) { Write-Output "verzögere Start um $StartDelay Sekunden" }
		Start-Sleep -s $StartDelay

		# starte VM
		if($verbose -eq $true) { Write-Output "Starte $VM" }
		Start-VM $MyVm -Server $Server -Force -Wait
	}
} else {
	if($verbose -eq $true) { Write-Output "TurnOff gesetzt" }
}


# ist KopiePath gesetzt dann kopiere und lösche export
if($RemotePath -ne "") {
	if(Test-Path $RemotePath\$VM) {
		if($verbose -eq $true) { Write-Output "Lösche Verzeichnis $RemotePath\$VM" }
		Remove-Item $RemotePath\$VM -recurse
	}
	#Remote oder Lokaler Export
	if($Server -ne "." ) {
		if($verbose -eq $true) { Write-Output "Kopiere $UNCPath\$VM nach $RemotePath\$VM (über diese Maschine)" }
		Copy-Item $UNCPath\$VM $RemotePath\$VM -Recurse
		if($verbose -eq $true) { Write-Output "Lösche Verzeichnis $UNCPath\$VM" }
		Remove-Item $UNCPath\$VM -recurse
	} else {
		if($verbose -eq $true) { Write-Output "Kopiere $ExportPath\$VM nach $RemotePath\$VM" }
		Copy-Item $ExportPath\$VM $RemotePath\$VM -Recurse
		if($verbose -eq $true) { Write-Output "Lösche Verzeichnis $ExportPath\$VM" }
		Remove-Item $ExportPath\$VM -recurse
	}
}

# Zeit ausgeben
if ($verbose -eq $true) { Write-Output (Get-Date) " Export der VM $vmName abgeschlossen" }